/*!
 @its2easy/animate-sprite 2.2.1
 https://github.com/its2easy/animate-sprite
         
 Copyright (c) 2020-present Dmitry Kovalev,
 Released under the MIT license
*/
/**
 * Returns validated frame number between the first and the last frame
 * @param {Number} frameNumber
 * @param {Number} totalFrames
 * @returns {Number}
 */
function normalizeFrameNumber(frameNumber, totalFrames){
    frameNumber = Math.floor(frameNumber);
    if (frameNumber <= 0) {
        return 1;
    } else if (frameNumber > totalFrames) {
        return totalFrames;
    }
    return frameNumber;
}

function capitalizeFirstLetter(string) {
    return string.charAt(0).toUpperCase() + string.slice(1);
}
function isNumeric(str) {
    return !isNaN(str) && // use type coercion to parse the _entirety_ of the string (`parseFloat` alone does not do this)...
        !isNaN(parseFloat(str)) // ...and ensure strings of whitespace fail
}

function validateParameters(node, options){
    if ( !(node instanceof HTMLElement || node instanceof Element || node instanceof HTMLDocument)) {
        throw new TypeError('Node is required');
    }
}
function validateSettings(options){
    ["width", "height", "frames"].forEach((el)=> {
        options[el] = parseInt(options[el], 10);
        if ( !isNumeric(options[el]) ) throw new TypeError(`options.${el} is required and must be a number`);
    });
    ["cols", "frameTime", "fps", "duration", "pageScrollTimerDelay"].forEach((el)=> {
        if (options[el] ) options[el] = parseInt(options[el], 10);
    });
    if ('dragModifier' in options) options.dragModifier = Math.abs(+options.dragModifier);
}

function getSettings(node, options){
    let inlineSettings = fillInlineSettings( node, getSettingsKeys() );
    return {...defaultSettings, ...options, ...inlineSettings};
}

const defaultSettings = {
    width: undefined,
    height: undefined,
    frames: undefined,
    cols: false,

    loop: false,
    autoplay: false,
    frameTime: false,
    duration: false,
    fps: 24,
    reverse: false,
    draggable: false,
    inversion: false,
    dragModifier: 1,
    touchScrollMode: "pageScrollTimer",
    pageScrollTimerDelay: 1500,
    onAnimationEnd: undefined,
    onAfterFrameChange: undefined,
};
function getSettingsKeys(){
    return Object.keys(defaultSettings);
}

/**
 * Returns object with data-* settings
 *
 * @param {HTMLElement} node
 * @param {Array<string>} list
 * @returns {Object}
 */
function fillInlineSettings(node, list) {
    let result = {};
    const prefix = "sprite";

    list.forEach( (value) => {
        const datasetFormattedValue = prefix + capitalizeFirstLetter(value);
        let inlineValue = node.dataset[datasetFormattedValue];

        if (typeof inlineValue !== 'undefined') {
            if ( inlineValue === "true" || inlineValue === '') inlineValue = true;
            else if ( inlineValue === "false") inlineValue = false;
            else if ( isNumeric(inlineValue) ) inlineValue = parseFloat(inlineValue);
            result[value] = inlineValue;
        }
    });
    return result;
}

class DragInput{
    static SWIPE_EVENTS = ['mousedown', 'mousemove', 'mouseup', 'touchstart', 'touchmove', 'touchend', 'touchcancel'];
    data;
    settings;
    changeFrame;
    getNextFrame;
    boundSwipeHandler;

    curX;
    curY;
    prevX;
    prevY;
    isSwiping = false;
    threshold;
    pixelsCorrection = 0;
    lastInteractionTime;

    constructor({ data, settings, changeFrame, getNextFrame }) {
        this.data = data;
        this.settings = settings;
        this.changeFrame = changeFrame;
        this.getNextFrame = getNextFrame;
        this.boundSwipeHandler = this.#swipeHandler.bind(this);
        this.updateThreshold();
    }

    enableDrag(){
        DragInput.SWIPE_EVENTS.forEach( (value) => {
            this.data.element.addEventListener(value, this.boundSwipeHandler);
        });
    }
    disableDrag(){
        DragInput.SWIPE_EVENTS.forEach( (value) => {
            this.data.element.removeEventListener(value, this.boundSwipeHandler);
        });
        this.#toggleDocumentEventHandlers(false);
        this.data.element.style.cursor = null;
    }

    /**
     * Update one frame threshold in pixels, based on DOM element width, should be called after nodeWidth change
     * @param newValue
     */
    updateThreshold(newValue = null){
        if (newValue) {
            this.threshold = newValue;
        }
        else {
            this.threshold = this.data.nodeWidth / this.settings.frames;
        }
    }
    #swipeHandler(event) {
        let touches;
        if ( event.touches !== undefined && event.touches.length ) touches = event.touches;
        this.curX = (touches) ? touches[0].pageX : event.clientX;
        this.curY = (touches) ? touches[0].pageY : event.clientY;

        switch (event.type){
            case 'mousedown':
            case 'touchstart':
                if ( event.type === 'touchstart' && event.cancelable) this.#maybeDisableScroll(event);
                this.#toggleDocumentEventHandlers(true);
                this.#swipeStart();
                break;
            case 'mousemove':
            case 'touchmove':
                if ( this.isSwiping ) this.#swipeMove();
                break;
            case 'mouseup':
            case 'touchend':
            case 'touchcancel':
                //if ( (event.type === 'touchend' || event.type === 'touchcancel') && event.cancelable) event.preventDefault();
                this.#toggleDocumentEventHandlers(false);
                this.#swipeEnd();
                break;
        }
    }
    #swipeStart(){
        this.data.pluginApi.stop();
        this.isSwiping = true;
        this.data.element.style.cursor = 'grabbing';
        this.prevX = this.curX;
        this.prevY = this.curY;
        this.data.element.dispatchEvent(new CustomEvent('sprite:drag-start',
            {detail: {frame: this.data.currentFrame}}
        ));
    }
    #swipeMove(){
        const direction = this.#swipeDirection();
        const swipeLength = Math.round( Math.abs(this.curX - this.prevX) * this.settings.dragModifier ) + this.pixelsCorrection;

        if ( swipeLength <= this.threshold) return; // Ignore if pixels are less than 1 frame
        if ( direction !== 'left' && direction !== 'right') return; // Ignore vertical directions

        this.prevX = this.curX;
        this.prevY = this.curY;

        const progress = swipeLength / this.data.nodeWidth;
        let deltaFrames = Math.floor(progress * this.settings.frames);
        deltaFrames = deltaFrames % this.settings.frames;
        // Add pixels to the next swipeMove if frames equivalent of swipe is not an integer number,
        // e.g one frame is 10px, swipeLength is 13px, we change 1 frame and add 3px to the next swipe,
        // so fullwidth swipe is always rotate sprite for 1 turn / all frames (with 'dragModifier' =  1)
        this.pixelsCorrection = swipeLength - (this.threshold * deltaFrames);
        let isReverse = (direction === 'left'); // left means backward (reverse: true)
        if (this.settings.inversion) isReverse = !isReverse;// invert direction
        this.changeFrame(this.getNextFrame( deltaFrames, isReverse ));
        this.data.element.dispatchEvent(new CustomEvent('sprite:drag-change',
            {detail: {frame: this.data.currentFrame}}
        ));
    }
    #swipeEnd(){
        this.curX = this.curY = this.prevX = this.prevY = null;
        this.isSwiping = false;
        this.data.element.style.cursor = null;
        this.lastInteractionTime = new Date().getTime();
        this.data.element.dispatchEvent(new CustomEvent('sprite:drag-end',
            {detail: {frame: this.data.currentFrame}}
        ));
    }
    #swipeDirection(){
        let xDist, yDist, r, swipeAngle;
        xDist = this.prevX - this.curX;
        yDist = this.prevY - this.curY;
        r = Math.atan2(yDist, xDist);
        swipeAngle = Math.round(r * 180 / Math.PI);
        if (swipeAngle < 0) swipeAngle = 360 - Math.abs(swipeAngle);
        if ( (swipeAngle >= 0 && swipeAngle <= 60) || (swipeAngle <= 360 && swipeAngle >= 300 )) return 'left';
        else if ( swipeAngle >= 120 && swipeAngle <= 240 ) return 'right';
        else if ( swipeAngle >= 241 && swipeAngle <= 299 ) return 'bottom';
        else return 'up';
    }
    /**
     * Idea from https://github.com/giniedp/spritespin/blob/master/src/plugins/input-drag.ts#L45
     * @param {Event} event
     */
    #maybeDisableScroll(event){
        // always prevent
        if (this.settings.touchScrollMode === "preventPageScroll") event.preventDefault();
        // check timer
        if (this.settings.touchScrollMode === "pageScrollTimer") {
            let now = new Date().getTime();
            // less time than delay => prevent page scroll
            if (this.lastInteractionTime && (now - this.lastInteractionTime < this.settings.pageScrollTimerDelay) ){
                event.preventDefault();
            } else { // more time than delay or first interaction => clear timer
                this.lastInteractionTime = null;
            }
        }
        // if touchScrollMode="allowPageScroll" => don't prevent scroll, so no action is needed
    }

    /**
     * Events to continue dragging if cursor is outside of the sprite block
     * @param {boolean} add
     */
    #toggleDocumentEventHandlers(add = true){
        if ( add ) {
            document.addEventListener('mouseup', this.boundSwipeHandler);
            document.addEventListener('mousemove', this.boundSwipeHandler);
        } else {
            document.removeEventListener('mouseup', this.boundSwipeHandler);
            document.removeEventListener('mousemove', this.boundSwipeHandler);
        }
    }
}

class Animation{
    settings;
    data;
    changeFrame;

    isAnimating = false;
    framesLeftToPlay = undefined; // frames from playTo() and playFrames()

    boundAnimate;
    lastUpdate; // time from RAF
    duration; // time of the full animation sequence
    stopRequested = false;

    framesQueue = 0; // save decimal part if deltaFrames is not round, to prevent rounding errors
    progressThreshold = 0.35; // >35% means that there was a long task in callstack

    constructor( {settings, data, changeFrame} ) {
        this.settings = settings;
        this.data = data;
        this.changeFrame = changeFrame;
        this.boundAnimate = this.#animate.bind(this);
        this.updateDuration();
    }

    play(){
        this.isAnimating = true;
        this.stopRequested = false; // fix for the case when stopRequested was set inside getNextFrame that was called outside #animate
        this.lastUpdate = null;// first 'lastUpdate' should be always set in the first raf of the current animation
        requestAnimationFrame(this.boundAnimate);
    }
    stop(){
        if ( this.isAnimating ){
            this.data.element.dispatchEvent( new Event('sprite:animation-end') );
            if ( this.settings.onAnimationEnd ) this.settings.onAnimationEnd(this.data.pluginApi);
        }
        this.isAnimating = false;
        this.framesLeftToPlay = undefined;
    }
    getNextFrame(deltaFrames, reverse = undefined){
        deltaFrames = Math.floor(deltaFrames); //just to be safe
        // Handle reverse
        if ( reverse === undefined ) reverse = this.settings.reverse;
        let newFrameNumber = reverse ? this.data.currentFrame - deltaFrames : this.data.currentFrame + deltaFrames;

        // Handle loop
        if (this.settings.loop) { // loop and outside of the frames
            if (newFrameNumber <= 0) {
                // for example newFrame = -2, total = 50, newFrame = 50 - abs(-2) = 48
                newFrameNumber = this.settings.frames - Math.abs(newFrameNumber);
            }
            else if (newFrameNumber > this.settings.frames) {
                // for example newFrame = 53, total 50, newFrame = newFrame - totalFrames = 53 - 50 = 3
                newFrameNumber = newFrameNumber - this.settings.frames;
            }
        } else { // no loop and outside of the frames
            if (newFrameNumber <= 0) {
                newFrameNumber = 1;
                this.stopRequested = true;
            }
            else if (newFrameNumber > this.settings.frames) {
                newFrameNumber = this.settings.frames;
                this.stopRequested = true;
            }
        }

        return  newFrameNumber;
    }

    // RAF callback
    // (chrome) 'timestamp' is timestamp from the moment the RAF callback was queued
    // (firefox) 'timestamp' is timestamp from the moment the RAF callback was called
    // the difference is equal to the time that the main thread was executing after raf callback was queued
    #animate(timestamp){
        if ( !this.isAnimating ) return; //stop() was called before this RAF

        // lastUpdate is setting here because the time between play() and #animate() is unpredictable, and
        // lastUpdate = performance.now() instead of timestamp because timestamp is unpredictable and depends on the browser.
        // Possible frame change in the first raf will always be skipped, because time <= performance.now
        if ( !this.lastUpdate) this.lastUpdate = performance.now();

        let deltaFrames;
        // Check if there was a long task between this and the last frame, if so move 1 fixed frame and change lastUpdate to now
        // to prevent animation jump. (1,2,3,long task,75,76,77, ... => 1,2,3,long task,4,5,6,...)
        // In this case the duration will be longer
        let isLongTaskBeforeRaf = (Math.abs(timestamp - performance.now()) / this.duration) > this.progressThreshold; //chrome check
        let progress = ( timestamp - this.lastUpdate ) / this.duration; // ex. 0.01
        if ( progress > this.progressThreshold ) isLongTaskBeforeRaf = true; // firefox check

        if (isLongTaskBeforeRaf) deltaFrames = 1; // raf after long task, just move to the next frame
        else { // normal execution, calculate progress after the last frame change
            if (progress < 0) progress = 0; //it happens sometimes, when raf timestamp is from the past for some reason
            deltaFrames = progress * this.settings.frames; // for example, 0.45 or 1.25
            // e.g. progress is 0.8 frames, queue is 0.25 frames, so now deltaFrames is 1.05 frames and we need to update canvas,
            // without this raf intervals will cause cumulative rounding errors, and actual fps will decrease
            deltaFrames = deltaFrames + this.framesQueue;
        }

        // calculate next frame only when we want to render
        // if the getNextFrame check was outside, getNextFrame would be called at screen fps rate, not animation fps
        // if screen fps 144 and animation fps 30, getNextFrame is calling now 30/s instead of 144/s.
        // After the last frame, raf is repeating until the next frame calculation,
        // between the last frame drawing and new frame time, reverse or loop could be changed, and animation won't stop
        if ( deltaFrames >= 1) { // Calculate only if we need to update 1 frame or more
            const newLastUpdate = isLongTaskBeforeRaf ? performance.now() : timestamp;

            this.framesQueue = deltaFrames % 1; // save decimal part for the next RAFs
            deltaFrames = Math.floor(deltaFrames) % this.settings.frames;
            if ( deltaFrames > this.framesLeftToPlay ) deltaFrames = this.framesLeftToPlay;// case when  animation fps > device fps
            const newFrame = this.getNextFrame( deltaFrames );
            if ( this.stopRequested ) { // animation ended from check in getNextFrame()
                this.data.pluginApi.stop();
                this.stopRequested = false;
                if (this.data.pluginApi.getCurrentFrame() !== newFrame ) this.changeFrame(newFrame); //last frame fix if fps > device fps
            } else { // animation is on
                this.lastUpdate = newLastUpdate;
                this.changeFrame(newFrame);
                if (typeof this.framesLeftToPlay !== 'undefined') {
                    this.framesLeftToPlay = this.framesLeftToPlay - deltaFrames;
                    // if 0 frames left, stop immediately, don't wait for the next frame calculation
                    // because if isAnimating become true, this will be a new animation
                    if ( this.framesLeftToPlay <= 0 ) this.data.pluginApi.stop();
                }
            }
        }
        if ( this.isAnimating ) requestAnimationFrame(this.boundAnimate);
    }
    updateDuration(){
        this.duration = this.#calculateDuration();
    }
    #calculateDuration(){
        if (this.settings.frameTime) return this.settings.frameTime * this.settings.frames; // frameTime
        else if (this.settings.duration) return this.settings.duration; // duration
        else return  ( this.settings.frames / this.settings.fps ) * 1000; // fps
    }
}

//todo core-js 3.21 adds es.error.cause(wasm polyfill?) to every file, current 3.20 is ok, check in the future
/**
 * Animate Sprite {@link https://github.com/its2easy/animate-sprite/}
 * @example
 * let sprite = new AnimateSprite( document.getElementById('sprite'), {
 *     width: 720,
 *     height: 405,
 *     frames: 20,
 *     cols: 5,
 *     fps: 60,
 * });
 */
class AnimateSprite {
    #settings;
    #data = {
        currentFrame: 1,
        nodeWidth: null,
        nodeHeight: null,
        element: null,
        pluginApi: {},
    }
    #boundCalculateSizes;
    // Classes
    #dragInput;
    #animation;

    /**
     * Creates plugin instance
     * @param {Element|HTMLElement} node - HTML element
     * @param {PluginOptions} options
     */
    constructor( node, options ) {
        validateParameters(node);
        this.#settings = getSettings(node, options);
        validateSettings(this.#settings);
        this.#data.element = node;
        this.#data.pluginApi = this;
        this.#boundCalculateSizes = this.#calculateSizes.bind(this);
        this.#initPlugin();
    }

    #initPlugin(){
        this.#animation = new Animation(
            {settings: this.#settings, data: this.#data, changeFrame: this.#changeFrame.bind(this)} );
        this.#calculateSizes();
        this.#toggleResizeHandler(true);

        if ( this.#settings.autoplay ) this.play();
        if ( this.#settings.draggable ) this.#toggleDrag(true);
    }

    #animateSprite(frame) {
        this.#data.element.style.backgroundPosition = this.#calculatePosition(frame);
        if ( this.#settings.onAfterFrameChange ) this.#settings.onAfterFrameChange(this.#data.pluginApi);
    }
    #changeFrame(frameNumber){
        if (frameNumber === this.#data.currentFrame) return;//skip same frame
        this.#animateSprite(frameNumber);
        this.#data.currentFrame = frameNumber;
    }
    #calculatePosition(frame){
        let xPadding, yPadding = 0;
        if ( !this.#settings.cols ){ // Single row sprite
            xPadding = (frame - 1) * this.#data.nodeWidth;
        } else { // Multiline sprite
            xPadding = ( ( (frame - 1) % this.#settings.cols)  ) * this.#data.nodeWidth;
            yPadding = Math.floor( (frame - 1) / this.#settings.cols ) * this.#data.nodeHeight;
        }
        return `-${xPadding}px -${yPadding}px`;
    }

    #calculateSizes(){
        const newNodeWidth = this.#data.element.offsetWidth;
        if (this.#data.nodeWidth === newNodeWidth ) return;//skip if same size

        const wasAnimating = this.#animation.isAnimating,
              widthHeightRatio = this.#settings.width / this.#settings.height;
        this.#data.nodeWidth = newNodeWidth;
        this.#data.nodeHeight = this.#data.nodeWidth / widthHeightRatio;
        this.#data.element.style.height = this.#data.nodeHeight + "px";

        let bgWidth =  ( !this.#settings.cols )
            ? this.#settings.frames * this.#data.nodeWidth
            : this.#settings.cols * this.#data.nodeWidth;
        let bgHeight = ( !this.#settings.cols )
            ? this.#data.nodeHeight
            : Math.ceil( this.#settings.frames / this.#settings.cols ) * this.#data.nodeHeight;
        this.#data.element.style.backgroundSize = `${bgWidth}px ${bgHeight}px`;
        if (wasAnimating) this.play();
            else this.#animateSprite(this.#data.currentFrame);
        if ( this.#dragInput ) this.#dragInput.updateThreshold();
    }

    #toggleDrag(enable = true){
        if (enable) {
            if ( !this.#dragInput ) this.#dragInput = new DragInput({
                data: this.#data,
                settings: this.#settings,
                changeFrame: this.#changeFrame.bind(this),
                getNextFrame: this.#animation.getNextFrame.bind(this.#animation)
            });
            this.#dragInput.enableDrag();
        } else {
            if (this.#dragInput) this.#dragInput.disableDrag();
        }
    }

    #toggleResizeHandler(add = true) {
        if ( add ) window.addEventListener("resize", this.#boundCalculateSizes);
        else window.removeEventListener("resize", this.#boundCalculateSizes);
    }

    // Public API

    /**
     * Start animation
     * @returns {AnimateSprite} - plugin instance
     */
    play(){
        if ( !this.#animation.isAnimating ) this.#animation.play();
        return this;
    }
    /**
     * Stop animation
     * @returns {AnimateSprite} - plugin instance
     */
    stop(){
        this.#animation.stop();
        return this;
    }
    /**
     * Toggle between start and stop
     * @returns {AnimateSprite} - plugin instance
     */
    toggle(){
        if (!this.#animation.isAnimating) this.play();
            else this.stop();
        return this;
    }
    /**
     * Show the next frame
     * @returns {AnimateSprite} - plugin instance
     */
    next(){
        this.stop();
        this.#changeFrame( this.#animation.getNextFrame(1) );
        return this;
    }
    /**
     * Show the previous frame
     * @returns {AnimateSprite} - plugin instance
     */
    prev(){
        this.stop();
        this.#changeFrame( this.#animation.getNextFrame(1, !this.#settings.reverse) );
        return this;
    }
    /**
     * Set frame (without animation)
     * @param {number} frameNumber - Number of the frame to show
     * @returns {AnimateSprite} - plugin instance
     */
    setFrame(frameNumber){
        this.stop();
        this.#changeFrame( normalizeFrameNumber(frameNumber, this.#settings.frames) );
        return this;
    }
    /**
     * Starts the animation, that plays until the specified frame number
     * @param {number} frameNumber - Target frame number
     * @param {Object} [options] - Options
     * @param {boolean} [options.shortestPath=false] - If set to true and loop enabled, will use the shortest path
     * @returns {AnimateSprite} - plugin instance
     */
    playTo(frameNumber, options){
        frameNumber = normalizeFrameNumber(frameNumber, this.#settings.frames);

        const innerPathDistance = Math.abs(frameNumber - this.#data.currentFrame), // not crossing edge frames
            outerPathDistance = this.#settings.frames - innerPathDistance, // crossing edges frames
            shouldUseOuterPath = this.#settings.loop && options?.shortestPath && (outerPathDistance < innerPathDistance);

        if ( !shouldUseOuterPath ) { // Inner path (default)
            // long conditions to make them more readable
            if (frameNumber > this.#data.currentFrame) this.setReverse(false); // move forward
            else this.setReverse(true); // move backward
        } else { // Outer path
            if (frameNumber < this.#data.currentFrame) this.setReverse(false); // move forward
            else this.setReverse(true); // move backward
        }

        return this.playFrames( (shouldUseOuterPath) ? outerPathDistance : innerPathDistance );
    }
    /**
     * Starts the animation in the current direction with the specified number of frames in queue
     * @param {number} [numberOfFrames=0] - Number of frames to play
     * @returns {AnimateSprite} - plugin instance
     */
    playFrames(numberOfFrames = 0){
        numberOfFrames = Math.floor(numberOfFrames);
        if (numberOfFrames < 0) return this.stop(); //empty animation

        this.#animation.framesLeftToPlay = numberOfFrames;
        return this.play();
    }
    /**
     * Change the direction of the animation. Alias to <b>setOption('reverse', true)</b>
     * @param {boolean} [reverse=true] True for backward animation, false for forward
     * @returns {AnimateSprite} - plugin instance
     */
    setReverse(reverse = true){
        this.#settings.reverse = !!reverse;
        return this;
    }
    /** Get current reverse option. Alias to <b>getOption('reverse')</b>
     * @returns {boolean} - reverse or not
     */
    getReverse() { return this.#settings.reverse; }
    /**
     * Calculate new dimensions (sprite element and frame), this function should be called if element size was changed by a script.
     * Called automatically after page resize
     * @returns {AnimateSprite} - plugin instance
     */
    updateSizes(){
        this.#calculateSizes();
        return this;
    }
    /**
     * Returns option value
     * @param {string} option - Option name. All options are allowed.
     * @returns {*} - Option value
     */
    getOption(option){
        const allowedOptions = getSettingsKeys();
        if (allowedOptions.includes(option)) {
            return this.#settings[option];
        } else {
            console.warn(`${option} is not allowed in getOption`);
        }
    }
    /**
     * Set new option value
     * @param {string} option - Option name. All options are allowed.
     * @param {*} value - new value
     * @returns {AnimateSprite} - plugin instance
     */
    setOption(option, value) {
        const allowedOptions = getSettingsKeys();
        if (allowedOptions.includes(option)) {
            this.#settings[option] = value;
            if ( option === "width" || option === "height" || option === "frames" || option === "cols" ) {
                this.#settings[option] = parseInt(value, 10); // additional validation
                this.#calculateSizes();
            }
            else if ( option === "frameTime" || option === "duration" || option === "fps" ) {
                this.#settings.frameTime = this.#settings.duration = this.#settings.fps = false; // Reset
                this.#settings[option] = parseInt(value, 10); // set again because of reset
                this.#animation.updateDuration();
            } else if ( option === "draggable" ) this.#toggleDrag(value);
            else if (option === 'dragModifier') this.#settings.dragModifier = Math.abs(+value);
        } else {
            console.warn(`${option} is not allowed in setOption`);
        }
        return this;
    }
    /** @returns {number} - current frame number */
    getCurrentFrame() { return this.#data.currentFrame; }
    /** @returns {boolean} - animating or not */
    isAnimating() { return this.#animation.isAnimating; }
    /**
     * Stop the animation and return to the first frame
     * @returns @returns {AnimateSprite} - plugin instance
     */
    reset(){
        this.stop();
        this.#changeFrame( normalizeFrameNumber(1, this.#settings.frames) );
        return this;
    }
    /**
     * Stop animation, remove event listeners. Method doesn't remove sprite element from the DOM
     */
    destroy() {
        this.stop();
        this.#toggleDrag(false);
        this.#toggleResizeHandler(false);
    }

}

// todo can't import typedef from another file because it won't add type to d.ts, check this in the future

/**
 * @typedef {object} PluginOptions
 * @property {number} width - Width of one frame in sprite
 * @property {number} height - Height of one frame in sprite
 * @property {number} frames - Number of frames
 * @property {number|false} [cols=false] - Number of cols if more than 1 row
 * @property {boolean} [loop=false] - Whether to loop the animation
 * @property {boolean} [autoplay=false] - Autoplay
 * @property {number|false} [frameTime] - ms, time between frames
 * @property {number|false} [duration] - ms, total time, alternative to frameTime
 * @property {number|false} [fps=24] - fps, alternative to frameTime
 * @property {boolean} [reverse=false] - Reverse direction of animation
 * @property {number|boolean} [draggable=false] - Draggable by mouse or touch
 * @property {boolean} [inversion=false] - Inversion changes drag direction
 * @property {number} [dragModifier=1] - Sensitivity factor for user interaction. Only positive numbers are allowed
 * @property {'pageScrollTimer' | 'preventPageScroll' | 'allowPageScroll'} [touchScrollMode="pageScrollTimer"]
 * - Page scroll behavior with touch events (preventPageScroll,allowPageScroll, pageScrollTimer)
 * @property {number} [pageScrollTimerDelay=1500] - Time in ms when touch scroll will be disabled during interaction
 * if options.touchScrollMode = "pageScrollTimer"
 * @property {function(AnimateSprite):void} [onAnimationEnd] - Occurs when animation has ended
 * @property {function(AnimateSprite):void} [onAfterFrameChange] - Occurs after the frame has switched
 */

export { AnimateSprite as default };
//# sourceMappingURL=animate-sprite.esm.js.map
